#############################################################################
## Some additional methods of wartslib classes.
##
## --------------------------------------------------------------------------
## Copyright (C) 2007-2008 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## $Id: wl-ping.rb,v 1.2 2008/10/05 05:08:53 youngh Exp $
#############################################################################

require 'yaml'

module Warts

class Ping

  private

  HALT_REASON = []
  HALT_REASON[Warts::Ping::STOP_NONE] = "S"
  HALT_REASON[Warts::Ping::STOP_COMPLETED] = "S"
  HALT_REASON[Warts::Ping::STOP_ERROR] = "E"

  public

  # from scamper_ping.h:SCAMPER_PING_STOP_*
  STOP_REASON_TEXT = Hash.new { |h, k| "<<UNKNOWN STOP CODE #{k}>>" }
  STOP_REASON_TEXT.merge!({
    Warts::Ping::STOP_NONE => "none (null reason)",
    Warts::Ping::STOP_COMPLETED => "completed (sent all probes)",
    Warts::Ping::STOP_ERROR => "error (sendto error)",
  })

  def to_yaml(limit_replies=false)
    value = {}
    value[".object"] = "ping"
    value[".dump_format"] = 1
    value["src"] = src
    value["dest"] = dest
    value["list_id"] = list_id
    value["cycle_id"] = cycle_id
    value["start"] = start
    value["start_usec"] = start_usec
    value["stop_reason"] = stop_reason
    #value["stop_reason_text"] = STOP_REASON_TEXT[stop_reason]
    value["stop_data"] = stop_data
    if pattern_bytes
      value["pattern_len"] = pattern_len
      value["pattern_bytes"] = pattern_bytes
    end
    value["attempts"] = attempts
    value["probe_size"] = probe_size
    value["wait"] = wait
    value["probe_ttl"] = probe_ttl
    value["tos"] = tos
    value["required_replies_count"] = required_replies_count
    value["sent_probes_count"] = sent_probes_count

    dest_reply = find_first_dest_reply()
    if dest_reply
      value["dest_replied"] = true
      value["dest_reply_rtt_sec"] = reply_rtt_sec(*dest_reply)
      value["dest_reply_rtt_usec"] = reply_rtt_usec(*dest_reply)
      value["dest_reply_probe"] = dest_reply[0]
      value["dest_reply_ttl"] = reply_reply_ttl(*dest_reply)
    else
      value["dest_replied"] = false
    end

    replies = []
    value["replies"] = replies
    each_probe do |probe, exists|
      if exists
        reply_value = {}
        replies << reply_value
        reply_value["probe_index"] = probe
	each_reply(probe) do |reply|
          reply_value["reply_index"] = reply
          reply_value["addr"] = reply_addr probe, reply 
          #reply_value["flags"] = reply_flags probe, reply
          if reply_has_reply_ttl? probe, reply
            reply_value["reply_ttl"] = reply_reply_ttl probe, reply 
          end
          reply_value["reply_size"] = reply_reply_size probe, reply
          reply_value["icmp_type"] = reply_icmp_type probe, reply
          reply_value["icmp_code"] = reply_icmp_code probe, reply
          reply_value["rtt_sec"] = reply_rtt_sec probe, reply
          reply_value["rtt_usec"] = reply_rtt_usec probe, reply
          reply_value["is_dest_reply"] = reply_dest_reply? probe, reply
	end
      end
    end

    YAML::dump value
  end


  def dump
    fields = ["P"]

    fields.push src, dest, list_id, cycle_id, start

    dest_reply = find_first_dest_reply()
    if dest_reply
      fields.push "R", reply_rtt_str(*dest_reply),
	dest_reply[0] + 1, reply_reply_ttl(*dest_reply)
    else
      fields.push "N", 0, 0, 0
    end

    fields.push(HALT_REASON[stop_reason] || ("?" + stop_reason.to_s))
    fields.push stop_data

    each_probe do |probe, exists|
      if exists
	probe_str = ""
	each_reply(probe) do |reply|
	  probe_str << ";" unless probe_str.empty?
	  probe_str << reply_addr(probe, reply)
	  probe_str << "," << reply_rtt_str(probe, reply)
	  probe_str << "," << reply_reply_ttl(probe, reply).to_s
	end

        fields.push probe_str
      else
        fields.push "q"
      end
    end

    fields.join("\t")
  end

end

end
